﻿//===========================================================================
//
//
//  CRYSTALFONTZ 
//
//  This code uses the 8-bit parallel MCU mode of the display.
//
//  https://www.crystalfontz.com/product/CFAF240320H1022T
//
//  The controller is a Sitronix ST7789V:
//    https://www.crystalfontz.com/controllers/Sitronix/ST7789V
//
//  Seeeduino v4.2, an open-source 3.3v capable Arduino clone.
//    https://www.seeedstudio.com/Seeeduino-V4.2-p-2517.html
//    https://github.com/SeeedDocument/SeeeduinoV4/raw/master/resources/Seeeduino_v4.2_sch.pdf
//
//	Demo code can also be found on GitHub:
//	  https://github.com/crystalfontz
//
//============================================================================
//
//
//
//===========================================================================
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//============================================================================



//============================================================================
// LCD & USD control lines
//   ARD      | Port |				  |  Function					| Wire
//------------+------+----------------+-----------------------------+------------
//  GND	      |		 | #1 #2 #16 #31  |  GROUND						| Black
//  3.3V      |		 | #3 #13-15 #29  |  POWER 3.3V					| Red
// -----------+------+----------------+-----------------------------+------------
// #14/A1     |  PC0 | #4			  |  Chip Enable Signal (CS)	| Purple		moving to pin 16
// #15/A2     |  PC1 | #5			  |  Data/Instruction   (RS)	| Yellow		moving to pin 17
// #16/A3     |  PC2 | #6			  |  Read/Write         (WR)	| Grey			moving to pin 18
// #17/A4     |  PC2 | #7			  |  Read/Write         (RD)	| Brown			moving to pin 19
//  #8/D8     |  PB0 | #28			  |  Reset		        (RESET) | Blue
// -----------+------+----------------+-----------------------------+------------
//  #0/D0     |  PD0 | #20			  |  LCD_D10 (DB0)				| Purple
//  #1/D1     |  PD1 | #21			  |  LCD_D11 (DB1)				| Green
//  #2/D2     |  PD2 | #22			  |  LCD_D12 (DB2)				| Orange
//  #3/D3     |  PD3 | #23			  |  LCD_D13 (DB3)				| Yellow
//  #4/D4     |  PD4 | #24			  |  LCD_D14 (DB4)				| Grey
//  #5/D5     |  PD5 | #25			  |  LCD_D15 (DB5)				| Brown
//  #6/D6     |  PD6 | #26			  |  LCD_D16 (DB6)				| White
//  #7/D7     |  PD7 | #27			  |  LCD_D17 (DB7)				| Black
// -----------+------+----------------+-----------------------------+------------
//============================================================================


//set the path of the controller
#include <SPI.h>
#include <SD.h>
#include <avr/io.h>

#include <SoftwareSerial.h>


//These defines are effectively fast versions of "digitalWrite(pin, 1/0)"
//using port manipulation and logic. CLR sets the pin to "0", SET sets it to "1".
#define CLR_CS     (PORTC &= ~(0x01)) //pin #14 - Chip Enable Signal
#define SET_CS     (PORTC |=  (0x01)) //pin #14 - Chip Enable Signal
#define CLR_RS     (PORTC &= ~(0x02)) //pin #15 - Data/Instruction
#define SET_RS     (PORTC |=  (0x02)) //pin #15 - Data/Instruction
#define CLR_WR	   (PORTC &= ~(0x04)) //pin #16 - Write
#define SET_WR	   (PORTC |=  (0x04)) //pin #16 - Write
#define CLR_RD	   (PORTC &= ~(0x08)) //pin #17 - Read
#define SET_RD	   (PORTC |=  (0x08)) //pin #17 - Read
#define CLR_RESET  (PORTB &= ~(0x01)) //pin #8  - Reset
#define SET_RESET  (PORTB |=  (0x01)) //pin #8  - Reset

//Colors are 16bit, here we define a few of them.
#define	BLACK	  0x0000
#define	BLUE	  0x001F
#define	RED 	  0xF800
#define	GREEN	  0x07E0
#define CYAN	  0x07FF
#define MAGENTA	  0xF81F
#define YELLOW	  0xFFE0
#define WHITE	  0xFFFF


//Since we're using ports 0&1 for the parallel bus, we can use SoftwareSerial to
//print out to the screen with the Analog pins A4 & A5 which is the same as D18 & D19
SoftwareSerial mySerial(18, 19);

void displayInit(void)
{
	//-----------------------------------------------------------
	//IMPORTANT: In order for this code to work with an SD card,
	//the Arduino must be power cycled after each new code upload
	//-----------------------------------------------------------




	//Set parallel interface port to output
	DDRD = 0xFF;
	//Drive all the pins low
	PORTD = 0x00;
	//Set PORTB and PORTC pins to input/output as necessary. 
	//SD and SoftwareSerial pins will be set by the class initialization
	DDRB = 0x10;
	DDRC = 0x2F;

	//Reset the module
	SET_RESET;
	delay(1);
	CLR_RESET;
	delay(10);
	SET_RESET;
	delay(120);
	CLR_CS;
	SET_RD;
	SET_WR;


	//--------------------------------ST7789S Frame rate setting----------------------------------// 
	//Porch Setting
	writeCommand(0xb2);
	//	  1011 0010
	writeData(0x0c);
	// 0000 1100
	// 0XXX XXXX
	//	||| ||||--	BPA6:BPA0
	//				  Back porch setting in normal mode. Minimum setting is 0x01

	writeData(0x0c);
	// 0000 1100
	// 0XXX XXXX
	//	||| ||||--	FPA6:FPA0
	//				  Front porch setting in normal mode. Minimum setting is 0x01

	writeData(0x00);
	// 0000 0000
	// 0000 000X
	//		   |--	PSEN
	//				>>0: Disable separate porch control
	//				  1: Disable separate porch control
	writeData(0x33);
	// 0011 0011
	// XXXX XXXX
	// |||| ||||--	FPB3:FPB0
	// ||||			  Front porch setting in idle mode. The minimum setting is 0x01
	// ||||
	// ||||-------	BPB3:BPB0
	//				  Back porch setting in idle mode. The minimum setting is 0x01
	writeData(0x33);
	// 0011 0011
	// XXXX XXXX
	// |||| ||||--	FPC3:FPC0
	// ||||			  Front porch setting in partial mode. The minimum setting is 0x01
	// ||||
	// ||||-------	BPC3:BPC0
	//				  Back porch setting in partial mode. The minimum setting is 0x01

	//Gate Control
	writeCommand(0xb7);
	//	  1011 0111
	writeData(0x35);
	// 0011 0101
	// 0XXX 0XXX
	//  |||	 |||--	VGL Setting
	//  |||			  000: -7.16
	//  |||			  001: -7.67
	//  |||			  010: -8.23
	//  |||			  011: -8.87
	//  |||			  100: -9.6
	//  |||			>>101: -10.43
	//  |||			  110: -11.38
	//  |||			  111: -12.5
	//  |||
	//  |||-------	VGH Setting
	//  			  000: 12.2
	//  			  001: 12.54
	//  			  010: 12.89
	//  			>>011: 13.26
	//  			  100: 13.65
	//  			  101: 14.06
	//  			  110: 14.5
	//  			  111: 14.97

	//---------------------------------ST7789S Power setting--------------------------------------//
	writeCommand(0x3a); // Interface Pixel Format
	writeData(0x05);	// 65K of RGB interface - 16bit/pixel
						// 	Bit	| Description                    | Comments
						//  -------------------------------------------------------------------
						// 	D7  | -                              | Set to �0�
						//  -------------------------------------------------------------------
						// 	D6  |                                | �101� = 65K of RGB interface
						// 	D5  | RGB interface color format     | �110� = 262K of RGB
						// 	D4  |                                | interface
						//  -------------------------------------------------------------------
						// 	D3  | -                              | Set to �0�
						//  -------------------------------------------------------------------
						// 	D2  |                                | �011� = 12bit/pixel
						// 	D1  | Control interface color format | �101� = 16bit/pixel
						// 	D0  |                                | �110� = 18bit/pixel
						// 	    |                                | �111� = 16M truncated
						//  -------------------------------------------------------------------

	writeCommand(0x36);	// Memory Data Access Control
	writeData(0x00);
	// 	Bit | NAME | DESCRIPTION
	//  -------------------------------------
	// 	D7  | MY   | Page Address Order
	// 	D6  | MX   | Column Address Order
	// 	D5  | MV   | Page/Column Order
	// 	D4  | ML   | Line Address Order
	// 	D3  | RGB  | RGB/BGR Order
	// 	D2  | MH   | Display Data Latch Order
	// 	D1  | -    | Don't Care
	// 	D0  | -    | Don't Care

	writeCommand(0xbb);	// VCOM Setting
	writeData(0x2b);
	// 	VCOMS[5:0] | VCOM (V) | VCOMS[5:0] | VCOM (V)
	//  ---------------------------------------------
	//  00h        | 0.1      | 20h        | 0.9
	//  01h        | 0.125    | 21h        | 0.925
	// 	02h        | 0.15     | 22h        | 0.95
	// 	03h        | 0.175    | 23h        | 0.975
	// 	04h        | 0.2      | 24h        | 1.0
	// 	05h        | 0.225    | 25h        | 1.025
	// 	06h        | 0.25     | 26h        | 1.05
	// 	07h        | 0.275    | 27h        | 1.075
	// 	08h        | 0.3      | 28h        | 1.1
	// 	09h        | 0.325    | 29h        | 1.125
	// 	0Ah        | 0.35     | 2Ah        | 1.15
	// 	0Bh        | 0.375    | 2Bh        | 1.175
	// 	0Ch        | 0.4      | 2Ch        | 1.2
	// 	0Dh        | 0.425    | 2Dh        | 1.225
	// 	0Eh        | 0.45     | 2Eh        | 1.25
	// 	0Fh        | 0.475    | 2Fh        | 1.275
	// 	10h        | 0.5      | 30h        | 1.3
	// 	11h        | 0.525    | 31h        | 1.325
	// 	12h        | 0.55     | 32h        | 1.35
	// 	13h        | 0.575    | 33h        | 1.375
	// 	14h        | 0.6      | 34h        | 1.4
	// 	15h        | 0.625    | 35h        | 1.425
	// 	16h        | 0.65     | 36h        | 1.45
	// 	17h        | 0.675    | 37h        | 1.475
	// 	18h        | 0.7      | 38h        | 1.5
	// 	19h        | 0.725    | 39h        | 1.525
	// 	1Ah        | 0.75     | 3Ah        | 1.55
	// 	1Bh        | 0.775    | 3Bh        | 1.575
	// 	1Ch        | 0.8      | 3Ch        | 1.6
	// 	1Dh        | 0.825    | 3Dh        | 1.625
	// 	1Eh        | 0.85     | 3Eh        | 1.65
	// 	1Fh        | 0.875    | 3Fh        | 1.675

	writeCommand(0xc2);
	writeData(0x01);

	writeCommand(0xc3);	// VRH Set
	writeData(0x0b);
	// VRHS[5:0] | VAP(GVDD) (V)                   | VRHS[5:0] | VAP(GVDD) (V)
	// -----------------------------------------------------------------------------------------
	// 00h       | 3.55+(vcom+vcom offset+0.5vdv)  | 15h       | 4.6+( vcom+vcom offset+0.5vdv)
	// 01h       | 3.6+( vcom+vcom offset+0.5vdv)  | 16h       | 4.65+( vcom+vcom offset+0.5vdv)
	// 02h       | 3.65+( vcom+vcom offset+0.5vdv) | 17h       | 4.7+( vcom+vcom offset+0.5vdv)
	// 03h       | 3.7+( vcom+vcom offset+0.5vdv)  | 18h       | 4.75+( vcom+vcom offset+0.5vdv)
	// 04h       | 3.75+( vcom+vcom offset+0.5vdv) | 19h       | 4.8+( vcom+vcom offset+0.5vdv)
	// 05h       | 3.8+( vcom+vcom offset+0.5vdv)  | 1Ah       | 4.85+( vcom+vcom offset+0.5vdv)
	// 06h       | 3.85+( vcom+vcom offset+0.5vdv) | 1Bh       | 4.9+( vcom+vcom offset+0.5vdv)
	// 07h       | 3.9+( vcom+vcom offset+0.5vdv)  | 1Ch       | 4.95+( vcom+vcom offset+0.5vdv)
	// 08h       | 3.95+( vcom+vcom offset+0.5vdv) | 1Dh       | 5+( vcom+vcom offset+0.5vdv)
	// 09h       | 4+( vcom+vcom offset+0.5vdv)    | 1Eh       | 5.05+( vcom+vcom offset+0.5vdv)
	// 0Ah       | 4.05+( vcom+vcom offset+0.5vdv) | 1Fh       | 5.1+( vcom+vcom offset+0.5vdv)
	// 0Bh       | 4.1+( vcom+vcom offset+0.5vdv)  | 20h       | 5.15+( vcom+vcom offset+0.5vdv)
	// 0Ch       | 4.15+( vcom+vcom offset+0.5vdv) | 21h       | 5.2+( vcom+vcom offset+0.5vdv)
	// 0Dh       | 4.2+( vcom+vcom offset+0.5vdv)  | 22h       | 5.25+( vcom+vcom offset+0.5vdv)
	// 0Eh       | 4.25+( vcom+vcom offset+0.5vdv) | 23h       | 5.3+( vcom+vcom offset+0.5vdv)
	// 0Fh       | 4.3+( vcom+vcom offset+0.5vdv)  | 24h       | 5.35+( vcom+vcom offset+0.5vdv)
	// 10h       | 4.35+( vcom+vcom offset+0.5vdv) | 25h       | 5.4+( vcom+vcom offset+0.5vdv)
	// 11h       | 4.4+( vcom+vcom offset+0.5vdv)  | 26h       | 5.45+( vcom+vcom offset+0.5vdv)
	// 12h       | 4.45+( vcom+vcom offset+0.5vdv) | 27h       | 5.5+( vcom+vcom offset+0.5vdv)
	// 13h       | 4.5+( vcom+vcom offset+0.5vdv)  | 28h~3Fh   | Reserved
	// 14h       | 4.55+( vcom+vcom offset+0.5vdv) | --        | --

	// VRHS[5:0] | VAN(GVCL) (V)                    | VRHS[5:0] | VAN(GVCL) (V)
	// -------------------------------------------------------------------------------------------
	// 00h       | -3.55+(vcom+vcom offset-0.5vdv)  | 15h       | -4.6+( vcom+vcom offset-0.5vdv)
	// 01h       | -3.6+( vcom+vcom offset-0.5vdv)  | 16h       | -4.65+( vcom+vcom offset-0.5vdv)
	// 02h       | -3.65+( vcom+vcom offset-0.5vdv) | 17h       | -4.7+( vcom+vcom offset-0.5vdv)
	// 03h       | -3.7+( vcom+vcom offset-0.5vdv)  | 18h       | -4.75+( vcom+vcom offset-0.5vdv)
	// 04h       | -3.75+( vcom+vcom offset-0.5vdv) | 19h       | -4.8+( vcom+vcom offset-0.5vdv)
	// 05h       | -3.8+( vcom+vcom offset-0.5vdv)  | 1Ah       | -4.85+( vcom+vcom offset-0.5vdv)
	// 06h       | -3.85+( vcom+vcom offset-0.5vdv) | 1Bh       | -4.9+( vcom+vcom offset-0.5vdv)
	// 07h       | -3.9+( vcom+vcom offset-0.5vdv)  | 1Ch       | -4.95+( vcom+vcom offset-0.5vdv)
	// 08h       | -3.95+( vcom+vcom offset-0.5vdv) | 1Dh       | -5+( vcom+vcom offset-0.5vdv)
	// 09h       | -4+( vcom+vcom offset-0.5vdv)    | 1Eh       | -5.05+( vcom+vcom offset-0.5vdv)
	// 0Ah       | -4.05+( vcom+vcom offset-0.5vdv) | 1Fh       | -5.1+( vcom+vcom offset-0.5vdv)
	// 0Bh       | -4.1+( vcom+vcom offset-0.5vdv)  | 20h       | -5.15+( vcom+vcom offset-0.5vdv)
	// 0Ch       | -4.15+( vcom+vcom offset-0.5vdv) | 21h       | -5.2+( vcom+vcom offset-0.5vdv)
	// 0Dh       | -4.2+( vcom+vcom offset-0.5vdv)  | 22h       | -5.25+( vcom+vcom offset-0.5vdv)
	// 0Eh       | -4.25+( vcom+vcom offset-0.5vdv) | 23h       | -5.3+( vcom+vcom offset-0.5vdv)
	// 0Fh       | -4.3+( vcom+vcom offset-0.5vdv)  | 24h       | -5.35+( vcom+vcom offset-0.5vdv)
	// 10h       | -4.35+( vcom+vcom offset-0.5vdv) | 25h       | -5.4+( vcom+vcom offset-0.5vdv)
	// 11h       | -4.4+( vcom+vcom offset-0.5vdv)  | 26h       | -5.45+( vcom+vcom offset-0.5vdv)
	// 12h       | -4.45+( vcom+vcom offset-0.5vdv) | 27h       | -5.5+( vcom+vcom offset-0.5vdv)
	// 13h       | -4.5+( vcom+vcom offset-0.5vdv)  | 28h~3Fh   | Reserved
	// 14h       | -4.55+( vcom+vcom offset-0.5vdv) | --        | --

	writeCommand(0xc4);	// VDV Setting
	writeData(0x20);
	// VDVS[5:0] | VDV (V) | VDVS[5:0] | VDV (V)
	//------------------------------------------
	// 00h       | -0.8    | 20h       | 0
	// 01h       | -0.775  | 21h       | 0.025
	// 02h       | -0.75   | 22h       | 0.05
	// 03h       | -0.725  | 23h       | 0.075
	// 04h       | -0.7    | 24h       | 0.1
	// 05h       | -0.675  | 25h       | 0.125
	// 06h       | -0.65   | 26h       | 0.15
	// 07h       | -0.625  | 27h       | 0.175
	// 08h       | -0.6    | 28h       | 0.2
	// 09h       | -0.575  | 29h       | 0.225
	// 0Ah       | -0.55   | 2Ah       | 0.25
	// 0Bh       | -0.525  | 2Bh       | 0.275
	// 0Ch       | -0.5    | 2Ch       | 0.3
	// 0Dh       | -0.475  | 2Dh       | 0.325
	// 0Eh       | -0.45   | 2Eh       | 0.35
	// 0Fh       | -0.425  | 2Fh       | 0.375
	// 10h       | -0.4    | 30h       | 0.4
	// 11h       | -0.375  | 31h       | 0.425
	// 12h       | -0.35   | 32h       | 0.45
	// 13h       | -0.325  | 33h       | 0.475
	// 14h       | -0.3    | 34h       | 0.5
	// 15h       | -0.275  | 35h       | 0.525
	// 16h       | -0.25   | 36h       | 0.55
	// 17h       | -0.225  | 37h       | 0.575
	// 18h       | -0.2    | 38h       | 0.6
	// 19h       | -0.175  | 39h       | 0.625
	// 1Ah       | -0.15   | 3Ah       | 0.65
	// 1Bh       | -0.125  | 3Bh       | 0.675
	// 1Ch       | -0.1    | 3Ch       | 0.7
	// 1Dh       | -0.075  | 3Dh       | 0.725
	// 1Eh       | -0.05   | 3Eh       | 0.75
	// 1Fh       | -0.025  | 3Fh       | 0.775

	writeCommand(0xc6);	// Frame Rate Control in Normal Mode
	writeData(0x0f);
	// NLA[2 :0] : Inversion selection in normal mode.
	// 0x00 : dot inversion.
	// 0x07: column inversion.

	// RTNA[4:0]:
	// RTNA[4:0] | FR in normal mode (Hz) | RTNA[4:0] | FR in normal mode (Hz)
	//------------------------------------------------------------------------
	// 00h       | 119                    | 10h       | 58
	// 01h       | 111                    | 11h       | 57
	// 02h       | 105                    | 12h       | 55
	// 03h       | 99                     | 13h       | 53
	// 04h       | 94                     | 14h       | 52
	// 05h       | 90                     | 15h       | 50
	// 06h       | 86                     | 16h       | 49
	// 07h       | 82                     | 17h       | 48
	// 08h       | 78                     | 18h       | 46
	// 09h       | 75                     | 19h       | 45
	// 0Ah       | 72                     | 1Ah       | 44
	// 0Bh       | 69                     | 1Bh       | 43
	// 0Ch       | 67                     | 1Ch       | 42
	// 0Dh       | 64                     | 1Dh       | 41
	// 0Eh       | 62                     | 1Eh       | 40
	// 0Fh       | 60                     | 1Fh       | 39

	writeCommand(0xd0);	// Power Control 1
	writeData(0xa4);	// 0xA4 Required
	writeData(0xa2);	// AVDD[1:0], AVCL[1:0], VDS[1:0]
						// AVDD[1:0] | AVDD (V)
						// 00h       | 6.4
						// 01h       | 6.6
						// 02h       | 6.8
						// 03h       | Reserved

						// AVCL[1:0] | AVCL (V)
						// 00h       | -4.4
						// 01h       | -4.6
						// 02h       | -4.8
						// 03h       | -5.0

						// VDS[1:0]  | VDDS (V)
						// 00h       | 2.19
						// 01h       | 2.3
						// 02h       | 2.4
						// 03h       | 2.51

						//--------------------------------ST7789S gamma setting---------------------------------------// 
	writeCommand(0xe0);	// Positive Voltage Gamma Control
						//See datasheet for more information
	writeData(0xd0);
	writeData(0x00);
	writeData(0x03);
	writeData(0x08);
	writeData(0x0a);
	writeData(0x17);
	writeData(0x2e);
	writeData(0x44);
	writeData(0x3f);
	writeData(0x29);
	writeData(0x10);
	writeData(0x0e);
	writeData(0x14);
	writeData(0x18);

	writeCommand(0xe1);	// Negative Voltage Gamma Control
						//See datasheet for more information
	writeData(0xd0);
	writeData(0x00);
	writeData(0x03);
	writeData(0x08);
	writeData(0x0a);
	writeData(0x17);
	writeData(0x2e);
	writeData(0x44);
	writeData(0x3f);
	writeData(0x29);
	writeData(0x10);
	writeData(0x0e);
	writeData(0x14);
	writeData(0x18);

	//Set the display window to the full size of the display
	setDisplayWindow(0x0000, 0x0000, 0x00EF, 0x013F);

	exitSleep();
}

void exitSleep(void)
{
	writeCommand(0x11); // Exit Sleep Mode
	delay(120);
	writeCommand(0x29); // Display on

	writeCommand(0x2c); // Memory write
						// When this command is accepted, the column register and the page
						// register are reset to the start column/start page positions.
}

void reset_display(void)
{
	CLR_RESET;
	delay(10);
	SET_RESET;
	delay(10);
	SET_RD;
}
//*********************************************************
void writeCommand(uint8_t command)
{
	CLR_RS; //CLR RS register - command

	PORTD = command & 0xFF;

	CLR_WR; //CLR write register
	SET_WR; //SET write register
}
//*********************************************************
void writeData(uint8_t data)
{
	SET_RS; //SET RS register - data

	PORTD = data & 0xFF;

	CLR_WR; //CLR write register
	SET_WR; //SET write register
}
//*********************************************************
uint8_t readData(void)
{
	uint8_t data;

	//SET RS register - data
	SET_RS;
	PORTD = 0x00; //Clear PORTE
	DDRD = 0x00; //Set PORTD pins to input
	CLR_RD; //CLR read register
	SET_RD; //SET read register
	delay(100);
	data = PIND; // read from the port
	DDRD = 0xFF; //Set PORTD pins to output
	delay(10);

	return data;
}
//********************************************************
void setDisplayWindow(int x0, int y0, int x1, int y1)
{
	writeCommand(0x2a);	// Column Address Set
	writeData(x0 >> 8);	// X address start:
	writeData(x0);		// 0 <= XS <= X
	writeData(x1 >> 8);	// X address end:
	writeData(x1);		// S <= XE <= X

	writeCommand(0x2b);	// Row Address Set
	writeData(y0 >> 8);	// Y address start:
	writeData(y0);		// 0 <= YS <= Y
	writeData(y1 >> 8);	// Y address start:
	writeData(y1);		// S <= YE <= Y

	//This command is necessary to ensure the addresses are set correctly, do not remove
	writeCommand(0x2c); // Memory write
}
// ********************************************************
void writeColor(uint16_t color)
{
	SET_RS; //SET RS register
	PORTD = color >> 8; //write the first 8 bytes of the color
	CLR_WR; //CLR write register
	SET_WR; //SET write register
	PORTD = color; //write the second 8 bytes of the color
	CLR_WR; //CLR write register
	SET_WR; //SET write register
}
// ********************************************************

void show_BMPs_in_root(void)
{
	File
		root_dir;
	root_dir = SD.open("/");
	if (0 == root_dir)
	{
		//Can't use debug console, all of port D is used by LCD
		mySerial.println("show_BMPs_in_root: Can't open \"root\"");
		//printLine("show_BMPs_in_root: Can't open \"root\"");
		return;
	}
	//set the display to start at 0,0;
	setDisplayWindow(0x0000, 0x0000, 0x00EF, 0x013F);

	File
		bmp_file;

	while (1)
	{
		bmp_file = root_dir.openNextFile();
		if (0 == bmp_file)
		{
			// no more files, break out of while()
			// root_dir will be closed below.
			break;
		}
		//Skip directories (what about volume name?)
		if (0 == bmp_file.isDirectory())
		{
			//The file name must include ".BMP"
			if (0 != strstr(bmp_file.name(), ".BMP"))
			{
				//The BMP must be exactly 230456 long
				//(this is correct for 240x320, 24-bit)
				if (230456 == bmp_file.size())
				{
					//Jump over BMP header. BMP must be 240x320 24-bit
					bmp_file.seek(54);

					//Since we are limited in memory, break the line up from
					// 240*3 = 720 bytes into three chunks of 80 pixels
					// each 80*3 = 240 bytes.
					//Making this static speeds it up slightly (10ms)
					//Reduces flash size by 114 bytes, and uses 240 bytes.
					static uint8_t
						third_of_a_line[80 * 3];
					for (int line = 319; line >= 0; line--)
					{
						//Set the LCD to the left of this line. BMPs store data
						//lowest line first -- bottom up.
						setDisplayWindow(0, line, 0x00EF, line);
						for (uint8_t line_section = 0; line_section<3; line_section++)
						{
							//Get a third of the line
							bmp_file.read(third_of_a_line, 80 * 3);
							//Now write this third to the TFT, doing the BGR -> RGB
							//color fixup interlaced with the SPI transfers.
							send_pixels(80 * 3, third_of_a_line);
						}
					}
				}
			}
		}
		//Release the BMP file handle
		bmp_file.close();
		//Give a bit to let them see it
		delay(1000);
	}
	//Release the root directory file handle
	root_dir.close();
}



void send_pixels(uint8_t byteCount, uint8_t *dataPtr)
{
	//To send pixels from a 24 bit BMP
	while (byteCount != 0)
	{
		//Pull first byte of the pixel and compress it to 5 bits
		uint16_t pixel = (*dataPtr >> 3) & 0x001F;
		dataPtr++; //point the pointer to the next byte
		byteCount--; //reduce the byteCount
		//Pull second byte of the pixel and compress it to 5 bits
		pixel |= ((*dataPtr << 3) & 0x07E0);
		dataPtr++;
		byteCount--;
		//Pull third and final byte of the pixel and compress it to 5 bits
		pixel |= (*dataPtr << 8) & 0xF800;
		dataPtr++;
		byteCount--;
		//write the color to the screen
		writeColor(pixel);
	}
}



void setup()
{
	mySerial.begin(9600);
	//Print out a blank block to the screen
	for (int i = 0; i < 100; i++)
	{
		mySerial.println();
	}
	mySerial.println("started setup");

	//Initialize the display
	displayInit();

	//An example of how to read the chip ID;
	mySerial.println("********CHIP ID*********");
	writeCommand(0x04);
	mySerial.print("dummy line: ");
	mySerial.println(readData(), HEX);
	mySerial.print("ID1 Read: ");
	mySerial.println(readData(), HEX);
	mySerial.print("ID2 Read: ");
	mySerial.println(readData(), HEX);
	mySerial.print("ID3 Read: ");
	mySerial.println(readData(), HEX);
	delay(5000);


	if (!SD.begin(10))
	{
		mySerial.println("Card failed to initialize, or not present");
		//Reset the SPI clock to fast. SD card library does not clean up well.
		//Bump the clock to 8MHz. Appears to be the maximum.
		SPI.beginTransaction(SPISettings(8000000, MSBFIRST, SPI_MODE0));
	}
	else
	{
		mySerial.println("Card initialized.");
	}

	mySerial.println("init completed");
}
void writeColorBars(void)
{
	unsigned int i, j;

	setDisplayWindow(0x0000, 0x0000, 0x00EF, 0x013F);

	//write 8 color bars. 320 / 8 = 40, so write a new color every 40 rows
	for (i = 0; i<320; i++)
	{
		for (j = 0; j<240; j++)
		{
			if (i>279)
			{
				writeColor(BLACK);
			}
			else if (i>239)
			{
				writeColor(BLUE);
			}
			else if (i>199)
			{
				writeColor(GREEN);
			}
			else if (i>159)
			{
				writeColor(CYAN);
			}
			else if (i>119)
			{
				writeColor(RED);
			}
			else if (i>79)
			{
				writeColor(MAGENTA);
			}
			else if (i>39)
			{
				writeColor(YELLOW);
			}
			else
			{
				writeColor(WHITE);
			}
		}
	}
}
void writeInverseColorBars(void)
{
	unsigned int i, j;

	setDisplayWindow(0x0000, 0x0000, 0x00EF, 0x013F);
	uint16_t row = 319;
	for (i = 0; i<320; i++)
	{
		//Since we're writing bottom up, each row we need to point to that row. 
		//The display is set to write top-left to bottom-right so we need to set
		//each row individually
		setDisplayWindow(0x0000, row, 0x00EF, row);
		row--;
		for (j = 0; j<240; j++)
		{
			if (i>279)
			{
				writeColor(BLACK);
			}
			else if (i>239)
			{
				writeColor(BLUE);
			}
			else if (i>199)
			{
				writeColor(GREEN);
			}
			else if (i>159)
			{
				writeColor(CYAN);
			}
			else if (i>119)
			{
				writeColor(RED);
			}
			else if (i>79)
			{
				writeColor(MAGENTA);
			}
			else if (i>39)
			{
				writeColor(YELLOW);
			}
			else
			{
				writeColor(WHITE);
			}
		}
	}
}


void fillScreen(unsigned long color)
{
	//Fill the screen with a color: set the window, write a color to each pixel
	unsigned int i, j;
	setDisplayWindow(0x0000, 0x0000, 0x00EF, 0x013F);

	for (i = 0; i<320; i++)
	{
		for (j = 0; j<240; j++)
		{
			writeColor(color);
		}
	}
}

void loop()
{
	fillScreen(WHITE);
	delay(1000);
	writeColorBars();
	delay(1000);
	writeInverseColorBars();
	delay(1000);
	show_BMPs_in_root();
	delay(1000);
}